// Modules to control application life and create native browser window
const {app,BrowserWindow,dialog} = require('electron');
const path = require('path');
const {ipcMain} = require('electron');
const fs = require("fs");
const prompt = require('electron-prompt');
const modHandler = require("./app/code/server/modHandler")
const modinfo = require("./app/code/mods/modinfo")

// Keep a global reference of the window object, if you don't, the window will
// be closed automatically when the JavaScript object is garbage collected.
var mainWindow;
var stage = 0;
var selectedProfile = "";
var dir = path.join(__dirname, "mods", "profiles");
var mHandler = new modHandler.ModHandler();

function createWindow() {
	// Create the browser window.
	mainWindow = new BrowserWindow({
		frame: false,
		height: 209,
		width: 600,
		webPreferences: {
			preload: path.join(__dirname, 'preload.js')
		}
	})

	// and load the index.html of the app.
	mainWindow.loadFile('app/splash.html');

	// Open the DevTools.
	// mainWindow.webContents.openDevTools()

	// Emitted when the window is closed.
	mainWindow.on('closed', function () {
		// Dereference the window object, usually you would store windows
		// in an array if your app supports multi windows, this is the time
		// when you should delete the corresponding element.
		if (stage == 0) {
			mainWindow = null;
		}
	})
}

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
app.on('ready', createWindow);

// Quit when all windows are closed.
app.on('window-all-closed', function () {
	// On macOS it is common for applications and their menu bar
	// to stay active until the user quits explicitly with Cmd + Q
	if (process.platform !== 'darwin') app.quit();
})

app.on('activate', function () {
	// On macOS it's common to re-create a window in the app when the
	// dock icon is clicked and there are no other windows open.
	if (mainWindow === null) createWindow();
})

// In this file you can include the rest of your app's specific main process
// code. You can also put them in separate files and require them here.
ipcMain.on("splashFinished", () => {
	// Profile Window
	let newWindow = CreateWindow({
		height: 440,
		width: 410,
		frame: true,
		webPreferences: {
			preload: path.join(__dirname, 'preload.js')
		}
	}, 1)
	mainWindow.loadFile('app/profiles.html');

	// Bind getProfiles to window call
	ipcMain.on('getProfiles', (e) => {
		fs.mkdirSync(dir, {
			recursive: true
		});
		mainWindow.webContents.send("getProfiles", fs.readdirSync(dir).filter((file) => {
			return fs.statSync(dir + '/' + file).isDirectory();
		}));
	});

	// Bind promptProfileCreation to window call
	ipcMain.on("promptProfileCreation", (e) => {
		prompt({
			height: 160,
			title: 'Enter a new profile name',
			label: 'Profile Name:',
			value: "",
			inputAttrs: {
				type: 'string'
			}
		}, mainWindow).then((r) => {
			if (!fs.existsSync(path.join(dir, r))) {
				fs.mkdirSync(path.join(dir, r), {
					recursive: true
				});
				mainWindow.webContents.send("getProfiles", fs.readdirSync(dir).filter((file) => {
					return fs.statSync(dir + '/' + file).isDirectory();
				}));
			}
		}).catch((e) => {
			console.log(e);
		})
	})

	ipcMain.on("deleteProfile", (e, profile) => {
		fs.rmdirSync(path.join(dir, profile));
		console.log("Deleting:", profile);
		mainWindow.webContents.send("getProfiles", fs.readdirSync(dir).filter((file) => {
			return fs.statSync(dir + '/' + file).isDirectory();
		}));
	});

	ipcMain.on("selectedProfile", (e, profile) => {
		selectedProfile = profile;
		startManagerWindow();
	});
});

function startManagerWindow() {
	let newWindow = CreateWindow({
		height: 350,
		width: 550,
		frame: true,
		webPreferences: {
			preload: path.join(__dirname, 'preload.js')
		}
	}, 2)
	mHandler.profile = path.join(dir, selectedProfile);
	ipcMain.on("getInstalledMods", (e, profile) => {
		mainWindow.webContents.send("getInstalledMods", mHandler.GetMods(mainWindow));
	});
	mainWindow.loadFile('app/manager.html');
}

ipcMain.on('resize', function (e, x, y) {
	mainWindow.setSize(x, y);
});

ipcMain.on('downloadMod', function (e, version, tsmod) {
	let progressWindow = new BrowserWindow({
		width: 300,
		height: 130,
		frame: true,
		webPreferences: {
			preload: path.join(__dirname, 'preload.js'),
			devTools: false,
		}
	});
	progressWindow.setMenu(null);
	progressWindow.setAlwaysOnTop(true, "floating");
	progressWindow.loadURL(path.join(__dirname, "app", "progress.html?modName=" + tsmod.full_name));
	mHandler.Download(tsmod, version, (res, message) => {
		progressWindow.close()
		if (!res) {
			mainWindow.webContents.send("downloadError", message);
		} else {
			// Create mod
			let mod = generateModFromManifest(message.extractLocation)
			mod.Uuid4 = tsmod.uuid4;
			mod.FullName = tsmod.full_name;
			mod.IsHTTP = true;
			mod.Path = message.extractLocation;
			mod.URL = tsmod.package_url;
			mod.Description = version.description;
			mod.Version = new modinfo.Version().ConvertFromString(version.version_number);
			mod.Author = tsmod.owner;
			mod.Icon = path.join(message.extractLocation, "icon.png");
			let modArray = [];
			modArray = JSON.parse(mHandler.GetMods(), true)
			insertOrReplaceMod(modArray, mod)
			let listString = JSON.stringify(modArray, null, 4)
			mHandler.Update(listString);
			mainWindow.webContents.send("getInstalledMods", listString);
		}
	});
});

function CreateWindow(opts, closeStage) {
	//opts.devTools = false
	let window = new BrowserWindow(opts);
	//window.setMenu(null);
	opts.icon = path.join(__dirname, "app", "assets", "r2.ico")
	window.on("close", function () {
		if (stage == closeStage) {
			mainWindow = null;
		}
	})
	stage += 1;
	old = mainWindow;
	mainWindow = window;
	old.close();
}

ipcMain.on("addLocalMod", function (e, x, y) {
	dialog.showOpenDialog(mainWindow, {
		title: "Select Mod to Install",
		filters: [{name: "zip", extensions: ["zip"]}],
		properties: ["openFile", "multiSelections"]
	}, (filePaths)=> {
		if (filePaths == null) {
			return;
		}
		let modArray = [];
		modArray = JSON.parse(mHandler.GetMods(), true)
		for (let i=0; i<filePaths.length; i++) {
			mHandler.Extract(filePaths[i], path.join(dir, selectedProfile, path.basename(filePaths[i], ".zip")), (valid)=>{
				if (valid) {
					let mod = generateModFromManifest(path.join(dir, selectedProfile, path.basename(filePaths[i], ".zip")))
					mod.Path = path.join(dir, selectedProfile, path.basename(filePaths[i], ".zip"))
					mod.Icon = path.join(mod.Path, "icon.png");
					insertOrReplaceMod(modArray, mod)
					let listString = JSON.stringify(modArray, null, 4)
					mHandler.Update(listString);
					mainWindow.webContents.send("getInstalledMods", listString);
				}
			})
		}
	});
});

function generateModFromManifest(directory) {
	let mod = new modinfo.Mod_V1()
	let buf = fs.readFileSync(path.join(directory, "manifest.json"), "utf8");
	buf.trim();
	if (!mod.FromManifest(buf)) {
		// is V2
		// TODO
	}
	return mod;
}

function insertOrReplaceMod(modArray, mod) {
	let foundInArray = false;
	modArray.forEach((value, key) => {
		if (value.FullName === mod.FullName) {
			foundInArray = true;
			modArray[key] = value;
		}
	})
	if (!foundInArray) {
		modArray.push(mod)
	}
	return modArray
}

ipcMain.on("playRoR2", function (e, x, y) {
	dialog.showOpenDialog(mainWindow, {
		title: "Select Mod to Install",
		filters: [{name: "zip", extensions: ["zip"]}],
		properties: ["openFile", "multiSelections"]
	}, (filePaths)=> {
		if (filePaths == null) {
			return;
		}
		let modArray = [];
		modArray = JSON.parse(mHandler.GetMods(), true)
		for (let i=0; i<filePaths.length; i++) {
			mHandler.Extract(filePaths[i], path.join(dir, selectedProfile, path.basename(filePaths[i], ".zip")), (valid)=>{
				if (valid) {
					let mod = generateModFromManifest(path.join(dir, selectedProfile, path.basename(filePaths[i], ".zip")))
					mod.Path = path.join(dir, selectedProfile, path.basename(filePaths[i], ".zip"))
					mod.Icon = path.join(mod.Path, "icon.png");
					insertOrReplaceMod(modArray, mod)
					let listString = JSON.stringify(modArray, null, 4)
					mHandler.Update(listString);
					mainWindow.webContents.send("getInstalledMods", listString);
				}
			})
		}
	});
});
